/**
 * Copyright(C) 2013 Patrik Dufresne Service Logiciel <info@patrikdufresne.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.planimod.core.planif;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.planimod.core.Employee;

/**
 * Adds a constrain for the preferred seniority Maximize the number of assigned employees with preferred seniority
 * employees.
 * 
 * @author Patrik Dufresne
 */
public class MaxPreferredSeniorities extends SolveRunnable {

    private static final transient Logger LOGGER = LoggerFactory.getLogger(MaxPreferredSeniorities.class);

    /**
     * Public constructor.
     * 
     * @param problem
     *            reference to ProblemBuilder
     */
    public MaxPreferredSeniorities(ProblemBuilder problem) {
        super(problem);
    }

    @Override
    protected boolean runSolver() throws GeneratePlanifException {
        List<Employee> employees = pb.getEmployees();
        for (int i = 0; i < employees.size(); i++) {
            // Avoid creating constraint for employees without preferred
            // seniority and required preferred team.
            if (employees.get(i) != null && employees.get(i).getPreferencialSeniority()) {
                MaxPreferredSeniority solver = new MaxPreferredSeniority(pb, i);
                solver.run();
            }
        }
        return true;
    }

    @Override
    protected boolean setUp() throws GeneratePlanifException {
        LOGGER.info("Create constraints to assign employees with " + //$NON-NLS-1$
                "preferred seniority."); //$NON-NLS-1$
        return true;
    }
}